
#define PROPERTY_COPYRIGHT     "Copyright  by iDiamond @ Forex Factory"

#define PROPERTY_VERSION       "23.262"                       // Year.Day of Year 

#define PROPERTY_DESCRIPTION_1 "Set the Symbol and/or the Period on a Chart."
#define PROPERTY_DESCRIPTION_2 ""
#define PROPERTY_DESCRIPTION_3 "Used to change the Chart Symbol to another Symbol, or to change the Chart Period to another Period."
#define PROPERTY_DESCRIPTION_4 "Symbols can be defined as specified input or can be taken from MarketWatch or the General List."
#define PROPERTY_DESCRIPTION_5 "This indicator can be displayed on the Main Chart Window or any Indicator Subwindow. Buttons can be set to any corner."
#define PROPERTY_DESCRIPTION_6 "Other options can be configured for button layout and appearance, or to set on all Charts."

#define DbugInfo(x)                                          // define as x for informational debugging 4

#ifdef __MQL4__
#define MQL4(x) x
#else
#define MQL4(x)
#endif

#ifdef __MQL5__
#define MQL5(x) x
#else
#define MQL5(x)
#endif

#include <Arrays\ArrayString.mqh>                             // CArrayString
#include <ChartObjects\ChartObjectsTxtControls.mqh>           // CChartObjectButton

#define LIST_DELIMITER " "
#define USD_MAJORS "AUDUSD EURUSD GBPUSD NZDUSD USDCAD USDCHF USDJPY"
#define EUR_MAJORS "EURAUD EURCAD EURCHF EURGBP EURJPY EURNZD EURUSD"
#define JPY_MAJORS "AUDJPY CADJPY CHFJPY EURJPY GBPJPY NZDJPY USDJPY"
#define MAJOR28 "AUDCAD AUDCHF AUDJPY AUDNZD AUDUSD CADCHF CADJPY CHFJPY EURAUD EURCAD EURCHF EURGBP EURJPY EURNZD EURUSD GBPAUD GBPCAD GBPCHF GBPJPY GBPNZD GBPUSD NZDCAD NZDCHF NZDJPY NZDUSD USDCAD USDCHF USDJPY"

#define MT4_PERIODS "M1 M5 M15 M30 H1 H4 D1 W1 MN"
#define MT5_PERIODS "M1 M2 M3 M4 M5 M6 M10 M12 M15 M20 M30 H1 H2 H3 H4 H6 H8 H12 D1 W1 MN"

enum ENUM_SYMBOL_LIST
{
    No_Symbols,                // No Symbols
    Input_Symbols,             // Input Symbol List
    Major_28_Symbols,          // Major 28 Symbols    
    MarketWatch_Symbols,       // MarketWatch Symbols
    All_General_List_Symbols,  // All General List Symbols
    USD_Major_Symbols,         // USD Major Symbols
    EUR_Major_Symbols,         // EUR Major Symbols
    JPY_Major_Symbols,         // JPY Major Symbols
};

enum ENUM_PERIOD_LIST
{
        No_Periods,      // No Periods
        Input_Periods,   // Input Period List
        All_MT4_Periods  // All MT4 Periods
#ifdef __MQL5__        
      , All_MT5_Periods  // All MT5 Periods
#endif 
};

#define INPUTVARDELIMITER               "================================"

sinput string Note_Symbols             = INPUTVARDELIMITER;   // Symbol List

sinput ENUM_SYMBOL_LIST SymbolList     = Input_Symbols;       // Which Symbol List
sinput string InputSymbols             = MAJOR28;             // Input_Symbols - Separate each by <Space>

sinput ENUM_PERIOD_LIST PeriodList     = No_Periods;          // Whick Period List
MQL4( sinput string InputPeriods       = MT4_PERIODS; )       // Input_Periods - Separate each by <Space>
MQL5( sinput string InputPeriods       = MT5_PERIODS; )       // Input_Periods - Separate each by <Space>

sinput string Note_Behaviour           = INPUTVARDELIMITER;   // Behaviour
sinput bool   bSetAllCharts            = false;               // Set Symbol/Period Change on All Charts

sinput string Note_Layout              = INPUTVARDELIMITER;   // Layout
sinput bool   bChartForeground         = false;               // Chart Foreground (Price overwrites buttons)
sinput int    iChartWindow             = 0;                   // Chart Window, 0=Main,1=Subwindow 1...
sinput ENUM_BASE_CORNER BaseCorner     = CORNER_LEFT_LOWER;   // Base Corner
sinput int    MarginX                  = 1;                   // Horizontal Margin from Corner
sinput int    MarginY                  = 1;                   // Vertical Margin from Corner
sinput int    ButtonsPerRow            = 14;                  // Number of Symbol Columns
sinput int    XButtonSpacing           = 0;                   // Horizontal Space Between Buttons
sinput int    YButtonSpacing           = 0;                   // Vertical Space Between Buttons
sinput int    SymbolPeriodSpacing      = 8;                   // Vertical Space Between Symbols and Periods
sinput bool   bTransparent             = false;               // Transparent Buttons
sinput long   ZOrder                   = 1;                   // ZOrder (Chart Click Priority)

sinput string Note_Button              = INPUTVARDELIMITER;   // Unselected Button Appearance
sinput color  ButtonColr               = clrWhite;            // Button Color
sinput color  ButtonBorderColr         = clrDarkGray;         // Button Border Color
sinput string sFontName                = "Calibri Light";     // Button Font Name
sinput int    iFontSize                = 8;                   // Button Font Size
sinput color  TextColr                 = clrBlack;            // Button Text Color

sinput string Note_ButtonSelected      = INPUTVARDELIMITER;   // Selected Button Appearance
sinput color  ButtonColrSelected       = clrDimGray;          // Button Color - Selected
sinput color  ButtonBorderColrSelected = clrDarkGray;         // Button Border Color - Selected
sinput string sFontNameSelected        = "Calibri Light";     // Button Font Name - Selected
sinput int    iFontSizeSelected        = 8;                   // Button Font Size - Selected
sinput color  TextColrSelected         = clrWhite;            // Button Text Color - Selected


const string         sTfTable[] = {"M1", "M2", "M3", "M4", "M5", "M6", "M10", "M12", "M15", "M20", "M30", "H1", "H2", "H3", "H4", "H6", "H8", "H12", "D1", "W1", "MN"};
const ENUM_TIMEFRAMES TfTable[] = {PERIOD_M1, PERIOD_M2, PERIOD_M3, PERIOD_M4, PERIOD_M5, PERIOD_M6, PERIOD_M10, PERIOD_M12, PERIOD_M15, PERIOD_M20, PERIOD_M30,
                                   PERIOD_H1, PERIOD_H2, PERIOD_H3, PERIOD_H4, PERIOD_H6, PERIOD_H8, PERIOD_H12, PERIOD_D1, PERIOD_W1, PERIOD_MN1};

string sUniquePrefix;

int ButtonsInARow;

CArrayString casSymbols;                // Symbols to use
CArrayString casPeriods;                // Periods to use
CChartObjectButton ObjButtonSymbols[];  // Symbol Button Objects
CChartObjectButton ObjButtonPeriods[];  // Period Button Objects


//
//
int OnInit()
{
    bool bResult;
    int iResult, Indx, nSymbols, nPeriods, nColumns = 0;
    uint FontFlags = FW_NORMAL, uiWidth, uiHeight, uiMaxWidthSymbol = 0, uiMaxHeightSymbol = 0, uiMaxWidthPeriod = 0, uiMaxHeightPeriod = 0;
    long lChartWinHandl;
    string sExeName;


    sExeName = MQLInfoString(MQL_PROGRAM_NAME);
    bResult = ChartGetInteger(0, CHART_WINDOW_HANDLE, 0, lChartWinHandl);
    sUniquePrefix = StringFormat("%s %lld.%d.%d[%d,%d] ", sExeName, lChartWinHandl, iChartWindow, BaseCorner, MarginX, MarginY);

    iResult = PrepareSymbols(&casSymbols);
    nSymbols = casSymbols.Total();
    iResult = PreparePeriods(&casPeriods);
    nPeriods = casPeriods.Total();

    if ((nColumns = MathMax(nSymbols, nPeriods)) < ButtonsPerRow)
      ButtonsInARow = nColumns;
      else ButtonsInARow = ButtonsPerRow;

    // Determine Maximum Symbol Button Sizes
    bResult = TextSetFont(sFontName, iFontSize * -10, FontFlags);
    for (Indx = 0; Indx < nSymbols; Indx++)
      {   // Get Max Width and Max Height of all Unselected Symbols using this Font and Size
          TextGetSize(" " + casSymbols.At(Indx) + " ", uiWidth, uiHeight);
          if (uiWidth > uiMaxWidthSymbol)
            uiMaxWidthSymbol = uiWidth;
          if (uiHeight > uiMaxHeightSymbol)
            uiMaxHeightSymbol = uiHeight;
      }
    bResult = TextSetFont(sFontNameSelected, iFontSizeSelected * -10, FontFlags);
    for (Indx = 0; Indx < nSymbols; Indx++)
      {   // Get Max Width and Max Height of all Selected Symbols using this Font and Size
          TextGetSize(" " + casSymbols.At(Indx) + " ", uiWidth, uiHeight);
          if (uiWidth > uiMaxWidthSymbol)
            uiMaxWidthSymbol = uiWidth;
          if (uiHeight > uiMaxHeightSymbol)
            uiMaxHeightSymbol = uiHeight;
      }

    // Determine Maximum Period Button Sizes
    bResult = TextSetFont(sFontName, iFontSize * -10, FontFlags);
    for (Indx = 0; Indx < nPeriods; Indx++)
      {   // Get Max Width and Max Height of all Unselected Periods using this Font and Size
          TextGetSize(" " + casPeriods.At(Indx) + " ", uiWidth, uiHeight);
          if (uiWidth > uiMaxWidthPeriod)
            uiMaxWidthPeriod = uiWidth;
          if (uiHeight > uiMaxHeightPeriod)
            uiMaxHeightPeriod = uiHeight;
      }
    bResult = TextSetFont(sFontNameSelected, iFontSizeSelected * -10, FontFlags);
    for (Indx = 0; Indx < nPeriods; Indx++)
      {   // Get Max Width and Max Height of all Selected Periods using this Font and Size
          TextGetSize(" " + casPeriods.At(Indx) + " ", uiWidth, uiHeight);
          if (uiWidth > uiMaxWidthPeriod)
            uiMaxWidthPeriod = uiWidth;
          if (uiHeight > uiMaxHeightPeriod)
            uiMaxHeightPeriod = uiHeight;
      }

    uiWidth  = MathMax(uiMaxWidthSymbol, uiMaxWidthPeriod);
    uiHeight = MathMax(uiMaxHeightSymbol, uiMaxHeightPeriod);
    iResult = CreateButtons(&casSymbols, ObjButtonSymbols, (int)uiWidth, (int)uiHeight,
                            &casPeriods, ObjButtonPeriods, (int)uiWidth, (int)uiHeight);    
    SetButtonSelections();

    return (INIT_SUCCEEDED);
} // OnInit() 


//
// 
int OnCalculate(const int rates_total, const int prev_calculated, const int begin, const double &price[])
{
    return(rates_total);
}


//
// id = Event ID from the ENUM_CHART_EVENT enumeration
//
void OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
    int iButton, nObjects;


    switch (id)
      {
          case CHARTEVENT_KEYDOWN:
          case CHARTEVENT_MOUSE_MOVE:
          case CHARTEVENT_MOUSE_WHEEL:
          case CHARTEVENT_OBJECT_CREATE: 
               break;
          case CHARTEVENT_OBJECT_CHANGE:
               break;
          case CHARTEVENT_OBJECT_DELETE:
          case CHARTEVENT_CLICK:
               break;
          case CHARTEVENT_OBJECT_CLICK:  // lparam = X coordinate, dparam = Y coordinate, sparam = Name of the graphical object
               nObjects = ArraySize(ObjButtonSymbols);
               for (iButton = 0; iButton < nObjects; iButton++)
                 {
                    if (ObjButtonSymbols[iButton].Name() == sparam)  // If button clicked
                      {   ChartSetSymbol(bSetAllCharts, ObjButtonSymbols[iButton].Description());  // Set/Change Symbol
                          break;
                      }
                 }

               nObjects = ArraySize(ObjButtonPeriods);
               for (iButton = 0; iButton < nObjects; iButton++)
                 {
                    if (ObjButtonPeriods[iButton].Name() == sparam)  // If button clicked
                      {   ChartSetPeriod(bSetAllCharts, ObjButtonPeriods[iButton].Description());  // Set/Change Period
                          break;
                      }
                 }
               break;
          case CHARTEVENT_OBJECT_DRAG:
          case CHARTEVENT_OBJECT_ENDEDIT:
               break;
          case CHARTEVENT_CHART_CHANGE:
               break;
          case CHARTEVENT_CUSTOM:
          case CHARTEVENT_CUSTOM_LAST:
               break;
      }
} // OnChartEvent()


//
//
void OnDeinit(const int reason)
{
    bool bResult;
    int nButtons, iButton;


    casSymbols.Clear();
    casPeriods.Clear();

    nButtons = ArraySize(ObjButtonSymbols);    
    for (iButton = 0; iButton < nButtons; iButton++)
      {   bResult = ObjButtonSymbols[iButton].Delete();
      }
    ArrayFree(ObjButtonSymbols);       

    nButtons = ArraySize(ObjButtonPeriods);    
    for (iButton = 0; iButton < nButtons; iButton++)
      {   bResult = ObjButtonPeriods[iButton].Delete();
      }
    ArrayFree(ObjButtonPeriods);

    ChartRedraw();
} // OnDeinit()


//
//
int PrepareSymbols(CArrayString* SymbolsArray)
{
    int Indx, nSymbols;


    SymbolsArray.Clear();

    if (SymbolList == No_Symbols)
      return(0);

    if ((SymbolList == Input_Symbols) || (SymbolList == Major_28_Symbols) || 
        (SymbolList == USD_Major_Symbols) || (SymbolList == EUR_Major_Symbols) || (SymbolList == JPY_Major_Symbols))
      {
          int nSubStrings;
          ushort uDelimiter;
          string sInputList = "", sResult[];

          switch (SymbolList)
            {   case Input_Symbols:      sInputList = InputSymbols;  break;
                case Major_28_Symbols:   sInputList = MAJOR28;       break;
                case USD_Major_Symbols:  sInputList = USD_MAJORS;    break;
                case EUR_Major_Symbols:  sInputList = EUR_MAJORS;    break;
                case JPY_Major_Symbols:  sInputList = JPY_MAJORS;    break;
            }

          uDelimiter = StringGetCharacter(LIST_DELIMITER, 0);  // Returns value of a symbol, located in the specified position of a string.          
          nSubStrings = StringSplit( sInputList, uDelimiter, sResult ); 
          for (Indx = 0; Indx < nSubStrings; Indx++)
            {   SymbolsArray.Add(sResult[Indx]);                               
            }
      }

    if ((SymbolList == All_General_List_Symbols) || (SymbolList == MarketWatch_Symbols))
      {
          bool bSelected;
          string sName;

          if (SymbolList == All_General_List_Symbols)
            bSelected = false;
            else bSelected = true;             // true = only symbols in MarketWatch
          nSymbols = SymbolsTotal(bSelected);  // Number of available (selected in Market Watch or all) symbols.

          for (Indx = 0; Indx < nSymbols; Indx++)
            {   sName = SymbolName(Indx, bSelected);  // Returns the name of a symbol, true = only symbols in MarketWatch 
                SymbolsArray.Add(sName);
            }
     }

    return( nSymbols = SymbolsArray.Total() );
} // PrepareSymbols()


//
//
int PreparePeriods(CArrayString* PeriodsArray)
{
    int Indx, nPeriods;


    PeriodsArray.Clear();

    if (PeriodList == No_Periods)
      return(0);

    if ( (PeriodList == Input_Periods) || (PeriodList == All_MT4_Periods) MQL5( || (PeriodList == All_MT5_Periods) ) )
      {
          int nSubStrings;
          ushort uDelimiter;
          string sInput = "", sResult[];

          uDelimiter = StringGetCharacter(LIST_DELIMITER, 0);  // Returns value of a symbol, located in the specified position of a string.

          switch (PeriodList)
            {         case Input_Periods:   sInput = InputPeriods;  break;
                      case All_MT4_Periods: sInput = MT4_PERIODS;   break;
                MQL5( case All_MT5_Periods: sInput = MT5_PERIODS;   break; )
            }
          nSubStrings = StringSplit(sInput, uDelimiter, sResult);
          for (Indx = 0; Indx < nSubStrings; Indx++)
            {   PeriodsArray.Add(sResult[Indx]);
            }
      }

    return( nPeriods = PeriodsArray.Total() );
} // PreparePeriods()


//
//  iWidth*, iHeight* specify Width/Height of each Button
//
int CreateButtons(CArrayString* SymbolsArray, CChartObjectButton& ButtonSymbols[], int iWidthSymbol, int iHeightSymbol,
                  CArrayString* PeriodsArray, CChartObjectButton& ButtonPeriods[], int iWidthPeriod, int iHeightPeriod)
{
    bool bResult;
    int iRow = 0, nRows, nButtonRows = 0, iCol, Xdistance, Ydistance;
    int nSymbols, nPeriods, Indx, iResult, iReturn = 0;
    string UniqueName;


    bResult = ChartSetInteger(0, CHART_FOREGROUND, bChartForeground);  // Price chart in the foreground

    nSymbols = casSymbols.Total();
    nRows = (nSymbols / ButtonsInARow);   // Div, Full Rows    
    if ((nSymbols % ButtonsInARow) > 0)   // Modulus Remainder, if a Partial Row
      nRows++;
    nButtonRows = nRows;

    nPeriods = casPeriods.Total();  
    nRows += (nPeriods / ButtonsInARow);  // Div, Full Rows
    if ((nPeriods % ButtonsInARow) > 0)   // Modulus Remainder, if a Partial Row
      nRows++;

    // Symbol Buttons
    ArrayFree(ObjButtonSymbols);
    iResult = ArrayResize(ObjButtonSymbols, nSymbols);
    for (Indx = 0; Indx < nSymbols; Indx++)
      {   UniqueName = sUniquePrefix + casSymbols[Indx] + "-" + IntegerToString(Indx);
          bResult = ObjButtonSymbols[Indx].Create(0, UniqueName, iChartWindow, 0, 0, 1, 1);

          bResult = ObjButtonSymbols[Indx].Description(casSymbols[Indx]);
          bResult = ObjButtonSymbols[Indx].SetInteger(OBJPROP_BACK,   bTransparent); 
          bResult = ObjButtonSymbols[Indx].SetInteger(OBJPROP_ZORDER, ZOrder);
          //bResult = ObjButtonSymbols[iSymbol].SetInteger(OBJPROP_HIDDEN, true); // Prohibit showing via terminal menu "Charts" - "Objects" - "List of objects". 
          bResult = ObjButtonSymbols[Indx].State(false);                       // button state (Pressed/Depressed) 
          //bResult = ObjButtonSymbols[iSymbol].Tooltip(casSymbols[iSymbol]);

          // Determine the Button Row. Start at Top if using a Top Corner, Reverse Order if using a Lower Corner 
          if ((BaseCorner == CORNER_LEFT_LOWER) || (BaseCorner == CORNER_RIGHT_LOWER))
            iRow = (nRows - 1) - (Indx / ButtonsInARow);
            else iRow = Indx / ButtonsInARow;  // Div
          //DbugInfo(Print(__FILE__, " ", __FUNCTION__, "@", __LINE__, " Indx=", Indx, " iRow=", iRow, " nRows=", nRows, " ButtonsInARow=", ButtonsInARow, " UnqiueName=", UniqueName); )
     
          // Determine the Button Column. Start at Left if using a Left Corner, Reverse Order if using a Right Corner 
          if ((BaseCorner == CORNER_RIGHT_LOWER) || (BaseCorner == CORNER_RIGHT_UPPER))
            iCol = (ButtonsInARow - 1) - (Indx % ButtonsInARow);
            else iCol = Indx % ButtonsInARow;  // Modulus Remainder

          Xdistance = MarginX + (iCol * (iWidthSymbol + XButtonSpacing));
          if ((BaseCorner == CORNER_RIGHT_LOWER) || (BaseCorner == CORNER_RIGHT_UPPER))
            Xdistance += iWidthSymbol;
          Ydistance = MarginY + (iRow * (iHeightSymbol + YButtonSpacing));
          if ((BaseCorner == CORNER_LEFT_LOWER) || (BaseCorner == CORNER_RIGHT_LOWER))
            Ydistance += iHeightSymbol + ((nSymbols > 0) && (nPeriods > 0) ? SymbolPeriodSpacing : 0);

          bResult = ObjButtonSymbols[Indx].Corner(BaseCorner);
          bResult = ObjButtonSymbols[Indx].X_Distance(Xdistance);
          bResult = ObjButtonSymbols[Indx].Y_Distance(Ydistance);
          bResult = ObjButtonSymbols[Indx].X_Size(iWidthSymbol);
          bResult = ObjButtonSymbols[Indx].Y_Size(iHeightSymbol);
      }
    iReturn += nSymbols;

    // Period Buttons
    ArrayFree(ObjButtonPeriods);
    iResult = ArrayResize(ObjButtonPeriods, nPeriods);
    for (Indx = 0; Indx < nPeriods; Indx++)
      {   UniqueName = sUniquePrefix + casPeriods[Indx] + "-" + IntegerToString(Indx);   
          bResult = ObjButtonPeriods[Indx].Create(0, UniqueName, iChartWindow, 0, 0, 1, 1);

          bResult = ObjButtonPeriods[Indx].Description(casPeriods[Indx]);
          bResult = ObjButtonPeriods[Indx].SetInteger(OBJPROP_BACK,   bTransparent); 
          bResult = ObjButtonPeriods[Indx].SetInteger(OBJPROP_ZORDER, ZOrder);
          //bResult = ObjButtonSymbols[iSymbol].SetInteger(OBJPROP_HIDDEN, true); // Prohibit showing via terminal menu "Charts" - "Objects" - "List of objects". 
          bResult = ObjButtonPeriods[Indx].State(false);                       // button state (Pressed/Depressed) 
          //bResult = ObjButtonSymbols[iSymbol].Tooltip(casSymbols[iSymbol]);

          // Determine the Button Row. Start at Top if using a Top Corner, Reverse Order if using a Lower Corner 
          if ((BaseCorner == CORNER_LEFT_LOWER) || (BaseCorner == CORNER_RIGHT_LOWER))
            iRow = (nRows - 1) - (Indx / ButtonsInARow) - nButtonRows;
            else iRow = (Indx / ButtonsInARow) + nButtonRows;  // Div + nButtonRows
          //DbugInfo(Print(__FILE__, " ", __FUNCTION__, "@", __LINE__, " Indx=", Indx, " iRow=", iRow, " nRows=", nRows, " ButtonsInARow=", ButtonsInARow, " UnqiueName=", UniqueName); )

          // Determine the Button Column. Start at Left if using a Left Corner, Reverse Order if using a Right Corner 
          if ((BaseCorner == CORNER_RIGHT_LOWER) || (BaseCorner == CORNER_RIGHT_UPPER))
            iCol = (ButtonsInARow - 1) - (Indx % ButtonsInARow);
            else iCol = Indx % ButtonsInARow;  // Modulus Remainder

          Xdistance = MarginX + (iCol * (iWidthPeriod + XButtonSpacing));
          if ((BaseCorner == CORNER_RIGHT_LOWER) || (BaseCorner == CORNER_RIGHT_UPPER))
            Xdistance += iWidthPeriod;
          Ydistance = MarginY + (iRow * (iHeightPeriod + YButtonSpacing));
          if ( ((BaseCorner == CORNER_LEFT_UPPER) || (BaseCorner == CORNER_RIGHT_UPPER)) && (nSymbols > 0) && (nPeriods > 0) ) 
            Ydistance += SymbolPeriodSpacing;          
          if ((BaseCorner == CORNER_LEFT_LOWER) || (BaseCorner == CORNER_RIGHT_LOWER))
            Ydistance += iHeightPeriod;

          bResult = ObjButtonPeriods[Indx].Corner(BaseCorner);
          bResult = ObjButtonPeriods[Indx].X_Distance(Xdistance);
          bResult = ObjButtonPeriods[Indx].Y_Distance(Ydistance);
          bResult = ObjButtonPeriods[Indx].X_Size(iWidthPeriod);
          bResult = ObjButtonPeriods[Indx].Y_Size(iHeightPeriod);
      }
    iReturn += nPeriods;

    return(iReturn);
} // CreateButtons()


//
//
void SetButtonSelections()
{
    int nButtons, Indx;
    string sTF;


    // Symbols Buttons
    nButtons = ArraySize(ObjButtonSymbols);
    for (Indx = 0; Indx < nButtons; Indx++)
      {
          if (ObjButtonSymbols[Indx].Description() == _Symbol)  // Current Chart Symbol
            {
                 ObjButtonSymbols[Indx].BackColor(ButtonColrSelected);
                 ObjButtonSymbols[Indx].BorderColor(ButtonBorderColrSelected);
                 ObjButtonSymbols[Indx].Color(TextColrSelected);
                 ObjButtonSymbols[Indx].Font(sFontNameSelected);
                 ObjButtonSymbols[Indx].FontSize(iFontSizeSelected);
                 continue;
            }

          // Non-selected Button
          ObjButtonSymbols[Indx].BackColor(ButtonColr);
          ObjButtonSymbols[Indx].BorderColor(ButtonBorderColr);
          ObjButtonSymbols[Indx].Color(TextColr);
          ObjButtonSymbols[Indx].Font(sFontName);
          ObjButtonSymbols[Indx].FontSize(iFontSize);
      }

    // Period Buttons
    nButtons = ArraySize(ObjButtonPeriods);
    for (Indx = 0; Indx < nButtons; Indx++)
      {
          if (ObjButtonPeriods[Indx].Description() == (sTF = TimeframeToString((ENUM_TIMEFRAMES)_Period)))  // Current Chart Period
            {
                 ObjButtonPeriods[Indx].BackColor(ButtonColrSelected);
                 ObjButtonPeriods[Indx].BorderColor(ButtonBorderColrSelected);
                 ObjButtonPeriods[Indx].Color(TextColrSelected);
                 ObjButtonPeriods[Indx].Font(sFontNameSelected);
                 ObjButtonPeriods[Indx].FontSize(iFontSizeSelected);
                 //DbugInfo(Print(__FILE__, " ", __FUNCTION__, "@", __LINE__, " Indx=", Indx, " sTF=", sTF); )
                 continue;
            }

          // Non-selected Button          
          ObjButtonPeriods[Indx].BackColor(ButtonColr);
          ObjButtonPeriods[Indx].BorderColor(ButtonBorderColr);
          ObjButtonPeriods[Indx].Color(TextColr);
          ObjButtonPeriods[Indx].Font(sFontName);
          ObjButtonPeriods[Indx].FontSize(iFontSize);
          //DbugInfo(Print(__FILE__, " ", __FUNCTION__, "@", __LINE__, " Indx=", Indx, " sTF=", sTF); )
      }
} // SetButtonSelections()


//
//
bool ChartSetSymbol(bool bAllCharts, string sSymbol)
{
    bool bResult = false;
    long chart_id = 0;
    ENUM_TIMEFRAMES enumTF;


    if (!bAllCharts)
      return( bResult = ChartSetSymbolPeriod(0, sSymbol, _Period) );  // Set/Change Symbol, Period remains the same

    chart_id = ChartNext(0);
    while (chart_id >= 0)
      {
          enumTF = ChartPeriod(chart_id);
          bResult = ChartSetSymbolPeriod(chart_id, sSymbol, enumTF);  // Set/Change Symbol, Period remains the same

          chart_id = ChartNext(chart_id);
      }

    return(bResult);
} // ChartSetSymbol()


//
//
bool ChartSetPeriod(bool bAllCharts, string sPeriod)
{
    bool bResult = false;
    long chart_id = 0;
    string sSymbol;
    ENUM_TIMEFRAMES tfPeriod;


    if ((tfPeriod = StringToTimeframe(sPeriod)) == PERIOD_CURRENT)
      return(false);

    if (!bAllCharts)
      return( bResult = ChartSetSymbolPeriod(0, _Symbol, tfPeriod) );  // Set/Change Period, Symbol remains the same

    chart_id = ChartNext(0);
    while (chart_id >= 0)
      {
          sSymbol = ChartSymbol(chart_id);
          bResult = ChartSetSymbolPeriod(chart_id, sSymbol, tfPeriod);  // Set/Change Period, Symbol remains the same

          chart_id = ChartNext(chart_id);
      }

    return(bResult);
} // ChartSetPeriod()


//
ENUM_TIMEFRAMES StringToTimeframe(string sTF)
{
    for (int i = ArraySize(sTfTable) - 1; i >= 0; i--)
      {  if (sTF == sTfTable[i])
           return(TfTable[i]);
      }
    return(PERIOD_CURRENT);  // PERIOD_CURRENT = 0
}


//
string TimeframeToString(ENUM_TIMEFRAMES tf)
{
    for (int i = ArraySize(TfTable) - 1; i >= 0; i--)
      {  if (tf == TfTable[i])
           return(sTfTable[i]);
      }
    return("");
}


